/* 
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:      qo1.c

   Purpose: To demonstrate how to solve a quadratic optimization
              problem using the MOSEK API.
 */
 
#include <stdio.h>

#include "mosek.h" /* Include the MOSEK definition file. */

#define NUMCON 1   /* Number of constraints.             */
#define NUMVAR 3   /* Number of variables.               */
#define NUMANZ 3   /* Number of non-zeros in A.           */
#define NUMQNZ 4   /* Number of non-zeros in Q.           */

static void MSKAPI printstr(void *handle,
                            char str[])
{
  printf("%s",str);
} /* printstr */

int main(int argc,char *argv[])
{

  double        c[]   = {0.0,-1.0,0.0};

  MSKboundkeye  bkc[] = {MSK_BK_LO};
  double        blc[] = {1.0};
  double        buc[] = {+MSK_INFINITY}; 
    
  MSKboundkeye  bkx[] = {MSK_BK_LO,
                         MSK_BK_LO,
                         MSK_BK_LO};
  double        blx[] = {0.0,
                         0.0,
                         0.0};
  double        bux[] = {+MSK_INFINITY,
                         +MSK_INFINITY,
                         +MSK_INFINITY};
  
  MSKlidxt      ptrb[] = {0, 1, 2 };
  MSKlidxt      ptre[] = {1, 2, 3};
  MSKidxt       asub[] = {0,   0,   0};
  double        aval[] = {1.0, 1.0, 1.0};
  
  MSKidxt       qsubi[NUMQNZ];
  MSKidxt       qsubj[NUMQNZ];
  double        qval[NUMQNZ];
  
  MSKidxt       j;
  double        xx[NUMVAR];

  MSKenv_t      env;
  MSKtask_t     task;
  MSKrescodee   r;
  
  /* Create the mosek environment. */
  r = MSK_makeenv(&env,NULL,NULL,NULL,NULL);

  /* Check whether the return code is ok. */
  if ( r==MSK_RES_OK )
  {
    /* Directs the log stream to the 'printstr' function. */
    MSK_linkfunctoenvstream(env,
                            MSK_STREAM_LOG,
                            NULL,
                            printstr);
  }

  /* Initialize the environment. */   
  r = MSK_initenv(env);
  if ( r==MSK_RES_OK )
  { 
  
  /* Create the optimization task. */
    r = MSK_maketask(env,NUMCON,NUMVAR,&task);

    if ( r==MSK_RES_OK )
    {
      r = MSK_linkfunctotaskstream(task,MSK_STREAM_LOG,NULL,printstr);
 
      if ( r==MSK_RES_OK )
      {
        r = MSK_inputdata(task,
                          NUMCON,NUMVAR,
                          NUMCON,NUMVAR,
                          c,0.0,
                          ptrb,
                          ptre,
                          asub,
                          aval,
                          bkc,
                          blc,
                          buc,
                          bkx,
                          blx,
                          bux);
      }

      if ( r==MSK_RES_OK )
      {
        /*
         * The lower triangular part of the Q
         * matrix in the objective is specified.
         */

        qsubi[0] = 0;   qsubj[0] = 0;  qval[0] = 2.0;
        qsubi[1] = 1;   qsubj[1] = 1;  qval[1] = 0.2;
        qsubi[2] = 2;   qsubj[2] = 0;  qval[2] = -1.0;
        qsubi[3] = 2;   qsubj[3] = 2;  qval[3] = 2.0;

        /* Input the Q for the objective. */

        r = MSK_putqobj(task,NUMQNZ,qsubi,qsubj,qval);
      }

      if ( r==MSK_RES_OK )
        r = MSK_optimize(task);

      if ( r==MSK_RES_OK )
      {
        MSK_getsolutionslice(task,
                             MSK_SOL_ITR,
                             MSK_SOL_ITEM_XX,
                             0,
                             NUMVAR,
                             xx);

        printf("Primal solution\n");
        for(j=0; j<NUMVAR; ++j)
          printf("x[%d]: %e\n",j,xx[j]);
      }
    }

    MSK_deletetask(&task);
  }
  MSK_deleteenv(&env);

  printf("Return code: %d\n",r);

  return ( r );
} /* main */
