package lo1;
 
/*
  Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

  File:      lo1.java

  Purpose:   Demonstrates how to solve a small linear
  optimization problem using the MOSEK Java API.
*/

import mosek.*;

class msgclass extends mosek.Stream {
    public msgclass ()
    {
        super ();
    }

    public void print (String msg)
    {
        System.out.print (msg);
    }
}

public class lo1
{
    static final int NUMCON = 3;
    static final int NUMVAR = 4;
    static final int NUMANZ = 9;

    public static void main (String[] args)
    {
        // Since the value infinity is never used, we define
        // 'infinity' symbolic purposes only
        double
            infinity = 0;
  
        double c[]    = {3.0, 1.0, 5.0, 1.0};

        int    ptrb[] = {0, 2, 5, 7};
        int    ptre[] = {2, 5, 7, 9};
        int    asub[] = { 0, 1,
                          0, 1, 2,
                          0, 1,
                          1, 2};
        double aval[] = { 3.0, 2.0,
                          1.0, 1.0, 2.0,
                          2.0, 3.0,
                          1.0, 3.0};
      
        int     bkc[]  = {mosek.Env.boundkey.fx,
                          mosek.Env.boundkey.lo,
                          mosek.Env.boundkey.up};
        double  blc[]  = {30.0,
                          15.0,
                          -infinity};
        double  buc[]  = {30.0,
                          +infinity,
                          25.0};
        
        int     bkx[]  = {mosek.Env.boundkey.lo,
                          mosek.Env.boundkey.ra,
                          mosek.Env.boundkey.lo,
                          mosek.Env.boundkey.lo};
        double  blx[]  = {0.0,
                          0.0,
                          0.0,
                          0.0};
        double  bux[]  = {+infinity,
                          10.0,
                          +infinity,
                          +infinity};
        
        double[] xx  = new double[NUMVAR];

        mosek.Env
            env = null;
        mosek.Task
            task = null;
        
        try
            {
                // Make mosek environment. 
                env  = new mosek.Env ();
                // Direct the env log stream to the user specified
                // method env_msg_obj.print
                msgclass env_msg_obj = new msgclass ();
//REF:enum:streamtype:log
                env.set_Stream (mosek.Env.streamtype.log,env_msg_obj);
                // Initialize the environment.
//REF:func:env:initenv
                env.init ();
                // Create a task object linked with the environment env.
                task = new mosek.Task (env, NUMCON, NUMVAR);
                // Directs the log task stream to the user specified
                // method task_msg_obj.print
                msgclass task_msg_obj = new msgclass ();
                task.set_Stream (mosek.Env.streamtype.log,task_msg_obj);
//REF:func:task:inputdata
                task.inputdata(NUMCON,NUMVAR,
                               c,
                               0.0,
                               ptrb,
                               ptre,
                               asub,
                               aval,
                               bkc,
                               blc,
                               buc,
                               bkx,
                               blx,
                               bux);
                /* A maximization problem */ 
//REF:func:task:putintparam
//REF:param:iparam:objective_sense
                task.putobjsense(mosek.Env.objsense.maximize);

                /* Solve the problem */

                try
                    {
//REF:func:task:optimize
                        task.optimize();
                    }
                catch (mosek.Warning e)
                    {
                        System.out.println ("Mosek warning:");
                        System.out.println (e.toString ());
                    }
//REF:func:task:solutionsummary
//REF:enum:streamtype:msg
                task.solutionsummary(mosek.Env.streamtype.msg);

//REF:func:task:getsolutionslice
//REF:enum:soltype:itr
//REF:enum:solitem:xx

                task.getsolutionslice(mosek.Env.soltype.bas, // Basic solution.     
                                      mosek.Env.solitem.xx,  // Which part of solution.
                                      0,      // Index of first variable.
                                      NUMVAR, // Index of last variable+1 
                                      xx);
                for(int j = 0; j < NUMVAR; ++j)
                    System.out.println ("x[" + j + "]:" + xx[j]);
            }
        catch (mosek.ArrayLengthException e)
            {
                System.out.println ("Error: An array was too short");
                System.out.println (e.toString ());
            }
        catch (mosek.Exception e)
            /* Catch both mosek.Error and mosek.Warning */
            {
                System.out.println ("An error or warning was encountered");
                System.out.println (e.getMessage ());
            }
        
        if (task != null) task.dispose ();
        if (env  != null)  env.dispose ();
    }
}
 
