package nltest;

/* 
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:      nltest.java

   Purpose:   Demonstrate how to use the functionality defined in
   scopt.java.
*/

import mosek.*;

class msgclass extends mosek.Stream {
    private boolean output_on;
    
    String prefix;

    public msgclass ( String prefix, boolean output_is_on ) 
    {
        super ();
        output_on = output_is_on;
        this.prefix = prefix;
    }
    
    public void print (String msg) 
    {
        if (output_on)
            System.out.print (prefix+msg);
    }
}

public class nltest
{

    static final int NUMOPRO = 1; /* Number of nonlinear expressions in the obj. */
    static final int NUMOPRC = 2; /* Number of nonlinear expressions in the con. */
    static final int NUMVAR  = 3; /* Number of variables.     */
    static final int NUMCON  = 2; /* Number of constraints.   */
    static final int NUMANZ  = 3; /* Number of nonzeros in A. */

    public static void main (String[] args) /*throws mosek.Error, mosek.Warning*/
    {
        mosek.Task
            task = null;
        mosek.Env
            env  = null;
        // Since the value infinity is never used, we define
        // 'infinity' symbolic purposes only
        double
            infinity = 0;

        try
            {
                env = new mosek.Env ();
                env.init ();
                task = new mosek.Task (env, 10,10);


        
                /* Purpose: Solves the problem

                minimize    x_1 - log(x_3)
                subject to  x_1^2 + x_2^2 <= 1
                x_2 + 2*x_2 - x_3 = 0
                x_3 >=0
                */
                StringBuffer buffer;

                double[] oprfo = new double[NUMOPRO];
                double[] oprgo = new double[NUMOPRO];
                double[] oprho = new double[NUMOPRO];
                double[] oprfc = new double[NUMOPRC];
                double[] oprgc = new double[NUMOPRC];
                double[] oprhc = new double[NUMOPRC];
                double[] c     = new double[NUMVAR];
                double[] aval  = new double[NUMANZ];
                double[] blc   = new double[NUMCON];
                double[] buc   = new double[NUMCON];
                double[] blx   = new double[NUMVAR];
                double[] bux   = new double[NUMVAR];

                int numopro, numoprc, numcon=NUMCON, numvar=NUMVAR;
                int[] opro  = new int[NUMOPRO];
                int[] oprjo = new int[NUMOPRO];
                int[] oprc  = new int[NUMOPRC];
                int[] opric = new int[NUMOPRC];
                int[] oprjc = new int[NUMOPRC];
                int[] aptrb = new int[NUMVAR];
                int[] aptre = new int[NUMVAR];
                int[] asub  = new int[NUMANZ];
                int[] bkc   = new int[NUMCON];
                int[] bkx   = new int[NUMVAR];

                

                /* Specify nonlinear terms in the objective. */
                numopro  = NUMOPRO;
                opro[0]  = Env.scopr.log; /* Defined in scopt.h */
                oprjo[0] = 2;
                oprfo[0] = -1.0;
                oprgo[0] = 1.0;  /* This value is never used. */
                oprho[0] = 0.0;

                /* Specify nonlinear terms in the constraints. */
                numoprc  = NUMOPRC;
  
                oprc[0]  = Env.scopr.pow;
                opric[0] = 0;
                oprjc[0] = 0;
                oprfc[0] = 1.0;
                oprgc[0] = 2.0;
                oprhc[0] = 0.0;

                oprc[1]  = Env.scopr.pow;
                opric[1] = 0;
                oprjc[1] = 1;
                oprfc[1] = 1.0;
                oprgc[1] = 2.0;
                oprhc[1] = 0.0;

                /* Specify c */
                c[0] = 1.0; c[1] = 0.0; c[2] = 0.0;
        
                /* Specify a. */
                aptrb[0] = 0;   aptrb[1] = 1;   aptrb[2] = 2;
                aptre[0] = 1;   aptre[1] = 2;   aptre[2] = 3;
                asub[0]  = 1;   asub[1]  = 1;   asub[2]  = 1;
                aval[0]  = 1.0; aval[1]  = 2.0; aval[2]  = -1.0;
        
                /* Specify bounds for constraints. */
                bkc[0] = mosek.Env.boundkey.up; 
                bkc[1] = mosek.Env.boundkey.fx;
                blc[0] = -infinity; 
                blc[1] = 0.0;
                buc[0] = 1.0;
                buc[1] = 0.0;
        
                /* Specify bounds for variables. */
                bkx[0] = mosek.Env.boundkey.fr;      
                bkx[1] = mosek.Env.boundkey.fr;
                bkx[2] = mosek.Env.boundkey.lo;
                blx[0] = -infinity;  
                blx[1] = -infinity; 
                blx[2] = 0.0;
                bux[0] = infinity;   
                bux[1] = infinity; 
                bux[2] = infinity;
                
                /* Set message stream callback */
                task.inputdata(numcon,numvar,
                               c,0.0,
                               aptrb,aptre,
                               asub,aval,
                               bkc,blc,buc,
                               bkx,blx,bux);

                /* Setup of nonlinear expressions. */
                task.putSCeval(opro,oprjo,oprfo,oprgo,oprho,
                               oprc,opric,oprjc,oprfc,oprgc,oprhc);

                /* Start solver */
                System.out.print ("Start optimizing\n");
                task.optimize();

                System.out.print ("Done optimizing\n");

                /* Print short summary of solution to the message stream */
                task.solutionsummary(mosek.Env.streamtype.msg);
            } 
        /* Catch any exceptions thrown by mosek */
        catch (mosek.Exception e) {
            System.out.println ("An error occurred: " + e.getMessage ());
            System.out.println (e.msg);
        } 
        /* Any other errors*/
        catch (java.lang.Exception e) {
            System.out.println (e.getMessage ());
        }

        if (task != null) task.dispose ();
        if (env  != null) env.dispose  ();

        System.out.println ("Finished");
    }
}


