package qo1;

/* 
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:      qo1.java

   Purpose:   Demonstrate how to solve a quadratic
              optimization problem using the MOSEK Java API.
 */

import mosek.*;

public class qo1 
{
    static final int NUMCON = 1;   /* Number of constraints.             */
    static final int NUMVAR = 3;   /* Number of variables.               */
    static final int NUMANZ = 3;   /* Number of numzeros in A.           */
    static final int NUMQNZ = 4;   /* Number of nonzeros in Q.           */
    public static void main (String[] args)
    {
        // Since the value infinity is never used, we define
        // 'infinity' symbolic purposes only
        double infinity = 0;
        double[] c = {0.0, -1.0, 0.0};

        int[]    bkc  = {mosek.Env.boundkey.lo};
        double[] blc = {1.0};
        double[] buc = {infinity};
        
        int[]  bkx   = {mosek.Env.boundkey.lo,
                        mosek.Env.boundkey.lo,
                        mosek.Env.boundkey.lo};        
        double[] blx = {0.0,
                        0.0,
                        0.0};
        double[] bux = {infinity,
                        infinity,
                        infinity};
        
        int[] ptrb = {0, 1, 2 };
        int[] ptre = {1, 2, 3};
        int[]    asub  = { 0,   0,   0 };
        double[] aval  = { 1.0, 1.0, 1.0 };

        double[] xx   = new double[NUMVAR];
  
        mosek.Env  
            env;
        mosek.Task 
            task;
         
        env  = new mosek.Env ();
        try
            {
                env.init ();
                
                task = new mosek.Task (env, NUMCON,NUMVAR);
        
                task.inputdata(NUMCON,NUMVAR,
                               c,0.0,
                               ptrb,
                               ptre,
                               asub,
                               aval,
                               bkc,
                               blc,
                               buc,
                               bkx,
                               blx,
                               bux);
                /*
                 * The lower triangular part of the Q
                 * matrix in the objective is specified.
                 */
            
                int[]    qsubi = {0,   1,    2,   2  };
                int[]    qsubj = {0,   1,    0,   2  };
                double[] qval =  {2.0, 0.2, -1.0, 2.0};
            
                /* Input the Q for the objective. */

//REF:func:task:putqobj
                  task.putqobj(qsubi,qsubj,qval);

                task.optimize();
                task.solutionsummary(mosek.Env.streamtype.msg);
                task.getsolutionslice(mosek.Env.soltype.itr,
                                      mosek.Env.solitem.xx,
                                      0,
                                      NUMVAR,
                                      xx);
            
                System.out.println ("Primal solution");
                for ( int j=0; j<NUMVAR; ++j )
                    System.out.println("x[" + j + "]: " + xx[j]);

                task.cleanup ();
                env.cleanup  ();
            }
        catch (mosek.ArrayLengthException e)
            /* Catch both Error and Warning */
            {
                System.out.println ("An error/warning was encountered");
                System.out.println (e.toString ());
            }
        catch (mosek.Exception e)
            {
                System.out.println ("An error/warning was encountered");
                System.out.println (e.msg);
            }
    } /* Main */
}
